#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <cutil.h>
#include <cutil_inline.h>

#include <vector>
#include <iostream>

using namespace std;

#include <thrust/sort.h>
#include <thrust/device_vector.h>

struct cpair
{
  float x;
  float y;
  __device__ __host__ inline bool operator < (const cpair & second) const
  {
    if (x < second.x)
      return true;
    if ((x == second.x) && (y < second.y))
      return true;
    return false;
  }
  friend ostream & operator<< (ostream & o, const cpair & p)
  {
    o << "(" << p.x << "," << p.y << ")";
    return o;
  }

};

void
fillarray (thrust::host_vector < cpair > &hv)
{
  srand (1234);

  for (unsigned int i = 0; i < hv.size (); i++)
    {
      hv[i].x = (float) rand () / RAND_MAX;
      hv[i].y = (float) rand () / RAND_MAX;
    }
}

int
main (int argc, char **argv)
{
  int N = 100000000;
  unsigned int hTimer;

  cutilCheckError (cutCreateTimer (&hTimer));


  if (argc > 1)
    N = atoi (argv[1]);

  printf ("Comparing CPU and GPU sort for %d elements\n", N);

  {

    thrust::device_vector < cpair > dv;
    {
      thrust::host_vector < cpair > hv (N);

      fillarray (hv);

      cutilCheckError (cutResetTimer (hTimer));
      cutilCheckError (cutStartTimer (hTimer));


      printf ("Creating device vector\n");
      dv = hv;
    }

    printf ("Sort start\n");

    thrust::sort (dv.begin (), dv.end ());

    cutilCheckError (cutStopTimer (hTimer));
    printf ("GPU time: %f msecs.\n", cutGetTimerValue (hTimer));

    for (unsigned int i = 0; i < 10; i++)
      cout << dv[i] << " ";
    cout << endl;
  }

  thrust::host_vector < cpair > hv (N);
  fillarray (hv);		// reset array

  cutilCheckError (cutResetTimer (hTimer));
  cutilCheckError (cutStartTimer (hTimer));

  thrust::sort (hv.begin (), hv.end ());

  cutilCheckError (cutStopTimer (hTimer));
  printf ("CPU time: %f msecs.\n", cutGetTimerValue (hTimer));


  for (unsigned int i = 0; i < 10; i++)
    cout << hv[i] << " ";
  cout << endl;

  cutilCheckError (cutDeleteTimer (hTimer));

  return 0;
}
